/*
 * js-linkedmap.js v1.0.0
 * (c) 2018-2019 Jesse Feng
 * Released under the MIT License.
 */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = global || self, global.LinkedMap = factory());
}(this, function () { 'use strict';

  /*
   * celia.js v6.1.1
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function isNil (value) {
    /* eslint eqeqeq: 0 */
    return value == null;
  }

  var isNil_1 = isNil;

  /*
   * celia.js v6.1.1
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function isNumber (value) {
    return typeof value === 'number';
  }

  var isNumber_1 = isNumber;

  /*
   * celia.js v6.1.1
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function isFunction (value) {
    return typeof value === 'function';
  }

  var isFunction_1 = isFunction;

  function isArrayLike (value) {
    return !isNil_1(value) && isNumber_1(value.length) && !isFunction_1(value);
  }

  var isArrayLike_1 = isArrayLike;

  /*
   * celia.js v6.1.1
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function bindContext (iterator, context) {
    return context ? iterator.bind(context) : iterator;
  }

  var _bindContext = bindContext;

  function forSlice (value, start, end, iterator, context) {
    iterator = _bindContext(iterator, context);
    for (var i = start; i < end; i++) {
      iterator(value[i], i, value) === false && (i = end);
    }
  }

  var _forSlice = forSlice;

  function forEach (value, iterator, context) {
    _forSlice(value, 0, value.length, iterator, context);
  }

  var _forEach = forEach;

  // import hasOwn from './hasOwn';

  function forOwn (value, iterator, context) {
    iterator = _bindContext(iterator, context);
    var keys = Object.keys(value);
    var len = keys.length;
    for (var i = 0, key = (void 0); i < len; i++) {
      key = keys[i];
      iterator(value[key], key, value) === false && (i = len);
    }
    // for (let key in value) {
    //   if (hasOwn(value, key) && iterator(value[key], key, value) === false) {
    //     break;
    //   }
    // }
  }

  var _forOwn = forOwn;

  function each (value, iterator, context) {
    !isNil_1(value) && _forOwn(value, iterator, context);
  }

  var forOwn$1 = each;

  /*
   * celia.js v6.1.1
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function append (arr, obj) {
    arr[arr.length] = obj;
    return arr;
  }

  var _append = append;

  /*
   * celia.js v6.1.1
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function isObject (value) {
    return value !== null && typeof value === 'object';
  }

  var isObject_1 = isObject;

  var isArray = Array.isArray;

  function alias (proto, mappings) {
    if (isObject_1(proto)) {
      forOwn$1(mappings, function (mapping, key) {
        key = proto[key];
        isArray(mapping)
          ? mapping.forEach(function (m) { return (proto[m] = key); })
          : (proto[mapping] = key);
      });
    }
  }

  var alias_1 = alias;

  var LinkedMap = function LinkedMap(iterator) {
    this
      .clear()
      .putAll(iterator);
  };

  LinkedMap.prototype._clone = function _clone (copy) {
    copy._link = this._link.slice(0);

    var copyMap = Object.create(null);
    forOwn$1(this._map, function (val, key) {
      copyMap[key] = val;
    });
    copy._map = copyMap;
    return copy;
  };

  /**
   * 自定义扩展获取 hash 值的方法
   * @param {any} key
   */
  LinkedMap.prototype._getHash = function _getHash (key) {
    return String(key);
  };

  LinkedMap.prototype.clear = function clear () {
    this._link = [];
    this._map = Object.create(null);
    return this;
  };

  LinkedMap.prototype.clone = function clone () {
    return this._clone(new LinkedMap());
  };

  LinkedMap.prototype.containsKey = function containsKey (key) {
    return (key in this._map);
  };

  LinkedMap.prototype.containsValue = function containsValue (value) {
    var ref = this;
      var _link = ref._link;
      var _map = ref._map;
    return _link.some(function (key) { return _map[key] === value; });
  };

  LinkedMap.prototype.forEach = function forEach (callback) {
    var ref = this;
      var _link = ref._link;
      var _map = ref._map;
    _forEach(_link, function (key) {
      return callback(_map[key], key);
    });
    return this;
  };

  LinkedMap.prototype.get = function get (key, defaultValue) {
    key = this._getHash(key);
    var value = this._map[key];
    return value === undefined
      ? defaultValue
      : value;
  };

  LinkedMap.prototype.isEmpty = function isEmpty () {
    return this._link.length === 0;
  };

  LinkedMap.prototype.keys = function keys (callback) {
    return callback
      ? _forEach(this._link, callback)
      : this._link.slice(0);
  };

  LinkedMap.prototype.put = function put (key, value) {
    key = this._getHash(key);
    var ref = this;
      var _link = ref._link;
      var _map = ref._map;
    !(key in _map) && _append(_link, key);
    _map[key] = value;
    return this;
  };

  LinkedMap.prototype.putAll = function putAll (iterator) {
      var this$1 = this;

    var i = 0;

    isArrayLike_1(iterator)
      ? _forEach(iterator, function (ref) {
          var key = ref[0];
          var value = ref[1];

        this$1.put(key, value);
        i++;
      })
      : forOwn$1(iterator, function (value, key) {
        this$1.put(key, value);
        i++;
      });
    return i;
  };

  LinkedMap.prototype.remove = function remove (key, specifiedValue) {
    var len = arguments.length;
    var result = false;
    var map = this._map;

    switch (len) {
      case 1:
        key = this._getHash(key);
        if (key in map) {
          delete map[key];
          result = true;
        }
        break;
      case 2:
        key = this._getHash(key);
        if ((key in map) && map[key] === specifiedValue) {
          delete map[key];
          result = true;
        }
        break;
      default:
        return false;
    }

    // 联动删除下标集合
    if (result) {
      var ref = this;
        var _link = ref._link;
      _link.splice(_link.indexOf(key), 1);
    }
    return result;
  };

  LinkedMap.prototype.replace = function replace (key, oldValue, newValue) {
    var len = arguments.length;
    var map = this._map;

    switch (len) {
      case 2:
        key = this._getHash(key);
        if ((key in map)) {
          map[key] = oldValue;
          return true;
        }
        break;
      case 3:
        key = this._getHash(key);
        if ((key in map) && map[key] === oldValue) {
          map[key] = newValue;
          return true;
        }
        break;
    }

    return false;
  };

  LinkedMap.prototype.size = function size () {
    return this._link.length;
  };

  LinkedMap.prototype.toString = function toString () {
    return '[object LinkedMap]';
  };

  LinkedMap.prototype.values = function values (callback) {
    var ref = this;
      var _link = ref._link;
      var _map = ref._map;
    return callback
      ? _forEach(_link, function (key, i) {
        return callback(_map[key], i);
      })
      : _link.map(function (key) { return _map[key]; });
  };

  var proto = LinkedMap.prototype;

  if (Symbol && Symbol.iterator) {
    proto[Symbol.iterator] = function () {
      var ref = this;
      var _link = ref._link;
      var _map = ref._map;
      var i = 0;
      var len = _link.length;
      return {
        next: function next() {
          var key = _link[i];
          return { value: [key, _map[key], i], done: ++i > len };
        }
      };
    };
  }

  alias_1(proto, {
    keys: 'keySet',
    put: 'set',
    remove: 'delete'
  });

  return LinkedMap;

}));
